Class: Instana::Exporter::Otlp::BaseConverter::SpanDataWithEvents

Inherits:
Struct
  • Object
show all
Extended by:
Forwardable
Defined in:
lib/instana/exporter/otlp/base_converter.rb

Overview

SpanData extended with an optional list of span events

We wrap the base struct to carry events without changing the positional keyword_init constructor used by all converters.

Instance Attribute Summary collapse

Instance Method Summary collapse

Instance Attribute Details

#span_dataObject

Returns the value of attribute span_data

Returns:

  • (Object)

    the current value of span_data



117
118
119
# File 'lib/instana/exporter/otlp/base_converter.rb', line 117

def span_data
  @span_data
end

#span_eventsObject

Returns the value of attribute span_events

Returns:

  • (Object)

    the current value of span_events



117
118
119
# File 'lib/instana/exporter/otlp/base_converter.rb', line 117

def span_events
  @span_events
end

Instance Method Details

#eventsObject



127
128
129
# File 'lib/instana/exporter/otlp/base_converter.rb', line 127

def events
  span_events
end

#total_recorded_eventsObject



131
132
133
# File 'lib/instana/exporter/otlp/base_converter.rb', line 131

def total_recorded_events
  span_events.size
end